type HttpMethod = 'GET' | 'POST' | 'PUT' | 'PATCH' | 'DELETE';

let instance: Api | null = null;

class Api {
  baseURL?: string = import.meta.env.VITE_BACKEND_URL;

  constructor() {
    if (instance) {
      return instance;
    }
    instance = this;
  }

  async request<T>(url: string, method: HttpMethod, body?: any, headers?: any): Promise<T> {
    const response = await fetch(`${this.baseURL}/api${url}`, {
      method,
      mode: 'cors',
      credentials: 'include',
      headers: {
        'Content-Type': 'application/json',
        'Access-Control-Allow-Origin': `${this.baseURL}, ${window.location.origin}`,
        'Access-Control-Allow-Credentials': 'true',
        ...headers,
      },
      body: JSON.stringify(body),
    });
  
    if (!response.ok) {
      throw new Error(`Request failed with status ${response.status}`);
    }
  
    const data = await response.json();
    return data as T;
  }
  
  async get<T>(url: string, headers?: any): Promise<T> {
    return this.request<T>(url, 'GET', undefined, headers);
  }
  
  async post<T>(url: string, body: any, headers?: any): Promise<T> {
    return this.request<T>(url, 'POST', body, headers);
  }
  
  async put<T>(url: string, body: any, headers?: any): Promise<T> {
    return this.request<T>(url, 'PUT', body, headers);
  }
  
  async del<T>(url: string, headers?: any): Promise<T> {
    return this.request<T>(url, 'DELETE', undefined, headers);
  }

  async patch<T>(url: string, body: any, headers?: any): Promise<T> {
    return this.request<T>(url, 'PATCH', body, headers);
  }
}

export const API = Object.freeze(new Api());
