import dotenv from 'dotenv';
import publicRouter from './routes/public';
import privateRouter from './routes/private';
import mongoose from 'mongoose';
import express from 'express';
import cors from 'cors';
import session, { SessionOptions } from 'express-session';
import MongoStore from 'connect-mongo';

dotenv.config();

const clientPromise = mongoose.connect(process.env.VITE_MONGO_URI!).then((mon) => mon.connection.getClient());
console.log('Connected to MongoDB at ' + process.env.VITE_MONGO_URI);

export const hour = 1000 * 60 * 60;
export const month = hour * 24 * 30;
export const cookieExpiration = process.env.NODE_ENV === 'production' ? month : hour;

const sessionOptions: SessionOptions = {
    secret: process.env.VITE_SECRET!,
    saveUninitialized: false, // don't create session until something stored
    resave: false, //don't save session if unmodified
    name: process.env.VITE_COOKIE_NAME!,
    cookie: {
        maxAge: cookieExpiration,
        httpOnly: true,
        secure: process.env.NODE_ENV === 'production',
        sameSite: process.env.NODE_ENV === 'production' ? 'none' : 'lax'
    },
    store: MongoStore.create({
        clientPromise,
        autoRemove: 'interval',
        autoRemoveInterval: 10,
        crypto: {
            secret: process.env.VITE_SECRET!
        }
    })
};

const app = express();

if (process.env.NODE_ENV === 'production') {
    app.set('trust proxy', 1); // trust first proxy
    sessionOptions.cookie!.secure = true; // serve secure cookies
}

const healthRouter = express.Router();
healthRouter.route('/healthcheck').get((_req, res) => {
    res.status(200).send('Hello there!');
});

app.use(cors({ credentials: true, origin: process.env.VITE_CLIENT_URL, exposedHeaders: ['set-cookie'] }));
app.use(express.json());
app.use(express.urlencoded({ extended: true }));
app.use(session(sessionOptions));
app.use(
    '/api',
    healthRouter,
    publicRouter,
    privateRouter
);

// For development mode, the server is started by Vite, so we only need to do this for production
if (process.env.NODE_ENV === 'production') {
    app.listen(3333, () => {
        console.log(`Listening at http://localhost:3333`);
    });
}

export const viteNodeApp = app;
